@@ -1,4 +1,9 @@
 package POE::Component::Client::HTTP::Request;
+BEGIN {
+  $POE::Component::Client::HTTP::Request::VERSION = '0.910';
+}
+# vim: ts=2 sw=2 expandtab
+
 use strict;
 use warnings;
 
@@ -6,6 +11,7 @@ use POE;
 
 use Carp;
 use HTTP::Status;
+use Errno qw(ETIMEDOUT);
 
 BEGIN {
   local $SIG{'__DIE__'} = 'DEFAULT';
@@ -39,6 +45,7 @@ use constant REQ_HISTORY       => 14;
 use constant REQ_START_TIME    => 15;
 use constant REQ_FACTORY       => 16;
 use constant REQ_CONN_ID       => 17;
+use constant REQ_PEERNAME      => 18;
 
 use constant RS_CONNECT        => 0x01; # establishing a connection
 use constant RS_SENDING        => 0x02; # sending request to server
@@ -60,7 +67,7 @@ sub import {
           REQ_ID REQ_POSTBACK REQ_CONNECTION REQ_HTTP_REQUEST REQ_STATE
           REQ_RESPONSE REQ_BUFFER REQ_OCTETS_GOT REQ_TIMER
           REQ_PROG_POSTBACK REQ_USING_PROXY REQ_HOST REQ_PORT
-          REQ_HISTORY REQ_START_TIME REQ_CONN_ID
+          REQ_HISTORY REQ_START_TIME REQ_CONN_ID REQ_PEERNAME
         )
       ) {
         no strict 'refs';
@@ -155,6 +162,7 @@ sub new {
     time(),             # REQ_START_TIME
     $factory,           # REQ_FACTORY
     undef,              # REQ_CONN_ID
+    undef,              # REQ_PEERNAME
   ];
   return bless $self, $class;
 }
@@ -425,11 +433,12 @@ sub check_redirect {
   # Make sure to frob any cookies set.  Redirect cookies are cookies, too!
   $self->[REQ_FACTORY]->frob_cookies($self->[REQ_RESPONSE]);
 
-  my $new_uri = $self->[REQ_RESPONSE]->header ('Location');
-  DEBUG and warn "REQ: Preparing redirect to $new_uri";
+  my $location_uri = $self->[REQ_RESPONSE]->header('Location');
+
+  DEBUG and warn "REQ: Preparing redirect to $location_uri";
   my $base = $self->[REQ_RESPONSE]->base();
-  $new_uri = URI->new($new_uri, $base)->abs($base);
-  DEBUG and warn "RED: Actual redirect uri is $new_uri";
+  $location_uri = URI->new($location_uri, $base)->abs($base);
+  DEBUG and warn "RED: Actual redirect uri is $location_uri";
 
   my $prev = $self;
   my $history = 0;
@@ -449,8 +458,18 @@ sub check_redirect {
     $newrequest->remove_header('Cookie');
 
     DEBUG and warn "RED: new request $newrequest";
-    $newrequest->uri($new_uri);
-    _set_host_header ($newrequest);
+    $newrequest->uri($location_uri);
+
+    # Don't change the Host header on a relative redirect.  This
+    # allows the HTTP::Request's Host to remain intact, per
+    # rt.cpan.org #63990.
+    if (defined $location_uri->scheme()) {
+      DEBUG and warn "RED: redirecting to absolute location $location_uri";
+      _set_host_header($newrequest);
+    }
+    else {
+      DEBUG and warn "RED: no new Host for relative redirect to $location_uri";
+    }
 
     $self->[REQ_STATE] = RS_REDIRECTED;
     DEBUG and warn "RED: new request $newrequest";
@@ -522,8 +541,8 @@ sub error {
 
   my $nl = "\n";
 
-  my $r = HTTP::Response->new($code);
-  my $http_msg = status_message ($code);
+  my $http_msg = status_message($code);
+  my $r = HTTP::Response->new($code, $http_msg, [ 'X-PCCH-Errmsg', $message ]);
   my $m = (
     "<html>$nl"
     . "<HEAD><TITLE>Error: $http_msg</TITLE></HEAD>$nl"
@@ -535,20 +554,28 @@ sub error {
     . "</HTML>$nl"
   );
 
-  $r->content ($m);
-  $r->request ($self->[REQ_HTTP_REQUEST]);
+  $r->content($m);
+  $r->request($self->[REQ_HTTP_REQUEST]);
   $self->[REQ_POSTBACK]->($r);
   $self->[REQ_STATE] |= RS_POSTED;
 }
 
 sub connect_error {
-  my ($self, $message) = @_;
+  my ($self, $operation, $errnum, $errstr) = @_;
 
   my $host = $self->[REQ_HOST];
   my $port = $self->[REQ_PORT];
 
-  $message = "Cannot connect to $host:$port ($message)";
-  $self->error (RC_INTERNAL_SERVER_ERROR, $message);
+  if ($operation eq "connect" and $errnum == ETIMEDOUT) {
+    $self->error(408, "Connection to $host:$port failed: timeout");
+  }
+  else {
+    $self->error(
+      RC_INTERNAL_SERVER_ERROR,
+      "Connection to $host:$port failed: $operation error $errnum: $errstr"
+    );
+  }
+
   return;
 }
 
@@ -577,6 +604,10 @@ __END__
 
 POE::Component::Client::HTTP::Request - an HTTP request class
 
+=head1 VERSION
+
+version 0.910
+
 =head1 SYNOPSIS
 
   # Used internally by POE::Component::Client::HTTP
@@ -751,4 +782,4 @@ or mail L<mailto:bug-POE-Component-Client-HTTP@rt.cpan.org>
 
 For questions, try the L<POE> mailing list (poe@perl.org)
 
-=cut
+=cut
\ No newline at end of file
